/* sysdefs.h -- #defines for your system created by ./support/mksysdefs.
   Do NOT EDIT this file, since any changes will disappear.
   Instead, edit ./support/mksysdefs, or config.h, or machines.h. */

#if !defined (_SYSDEFS_H_)
#  define _SYSDEFS_H_

#if !defined (RANLIB_LOCATION)
#  define RANLIB_LOCATION ranlib
#endif /* RANLIB_LOCATION */

#if !defined (HAVE_DEV_FD)
#  define HAVE_DEV_FD
#endif /* HAVE_DEV_FD */

#endif /* _SYSDEFS_H_ */
